#### Înregistrări de resurse

Fiecărui domeniu, fie că este un singur calculator gazdă, fie un domeniu
de nivel superior, îi poate fi asociată o mulțime de înregistrări de
resurse (resource records sau RR-uri). Pentru un singur sistem gazdă,
cea mai obișnuită înregistrare de resursă este chiar adresa IP, dar
există multe alte tipuri.

Atunci când procedura resolver trimite un nume de domeniu DNS, ceea ce
va primi ca răspuns sunt înregistrările de resurse asociate acelui nume.
Astfel, adevărata funcție a DNS este să realizeze corespondența dintre
numele de domenii și înregistrări de resurse.

O înregistrare de resursă este un 5-tuplu. Cu toate că, din rațiuni de
eficiență, înregistrările de resurse sunt codificate binar, în
majoritatea expunerilor ele sunt prezentate ca text ASCII, câte o
înregistrare de resurse pe linie. Formatul utilizat este
*\<Nume_domeniu, Timp_de_viață, Tip, Clasă, Valoare>*:

-   Câmpul *Nume_domeniu* precizează domeniul căruia i se aplică această
    înregistrare. În mod normal, există mai multe înregistrări pentru
    fiecare domeniu, și fiecare copie a bazei de date păstrează
    informații despre mai multe domenii. Acest câmp este utilizat cu rol
    de cheie de căutare primară pentru a satisface cererile. Ordinea
    înregistrărilor în baza de date nu este semnificativă. Când se face
    o interogare despre un domeniu, sunt returnate toate înregistrările
    care se potrivesc cu clasa cerută.
-   Câmpul *Timp_de_viață* dă o indicație despre cât de stabilă este
    înregistrarea.
-   Câmpul *Tip* precizează tipul înregistrării. Cele mai importante
    tipuri sunt prezentate în tabelul de mai jos.

| Tip   | Semnificație                    | Valoare                                                 |
|-------|---------------------------------|---------------------------------------------------------|
| SOA   | Start autoritate                | Parametri pentru această zonă                           |
| A     | Adresa IPv4 a unui sistem gazdă | Întreg pe 32 de biți                                    |
| AAAA  | Adresa IPv6 a unui sistem gazdă | Întreg pe 128 de biți                                   |
| MX    | Schimb de poștă                 | Prioritate, domeniu dispus să accepte poștă electronică |
| NS    | Server de nume                  | Numele serverului pentru acest domeniu                  |
| CNAME | Nume canonic                    | Numele domeniului                                       |
| PTR   | Pointer                         | Pseudonim pentru adresă IP                              |
| HINFO | Descriere sistem gazdă          | Unitate centrală și sistem de operare în ASCII          |
| TXT   | Text                            | Text ASCII neinterpretat                                |

O înregistrare SOA furnizează numele sursei primare de informație despre
zona serverului de nume, adresa de e-mail a administratorului, un
identificator unic si diverși indicatori și contoare de timp.

Cel mai important tip de înregistrare este înregistrarea A (adresă). Ea
păstrează adresa IP de 32 de biți a sistemului gazdă. Următoarea ca
importanță este înregistrarea MX. Aceasta precizează numele domeniului
pregătit să accepte poștă electronică pentru domeniul specificat.
Înregistrările specifică numele serverului. 

Un exemplu de informație ce se poate găsi în baza de date DNS a unui domeniu este următorul:

    ; Authoritative Information on physics.groucho.edu.
    @ IN SOA niels.physics.groucho.edu.
    janet.niels.physics.groucho.edu. {
    1999090200 ; serial no
    360000 ; refresh
    3600 ; retry
    3600000 ; expire
    3600 ; default ttl
    };
    ; Name servers
    IN NS niels
    IN NS gauss.maths.groucho.edu.
    gauss.maths.groucho.edu. IN A 149.76.4.23
    ;
    ; Theoretical Physics (subnet 12)
    niels IN A 149.76.12.1
    IN A 149.76.1.12
    name server IN CNAME niels
    otto IN A 149.76.12.2
    quark IN A 149.76.12.4
    down IN A 149.76.12.5
    strange IN A 149.76.12.6
    ...
    ; Collider Lab. (subnet 14)
    boson IN A 149.76.14.1
    muon IN A 149.76.14.7
    bogon IN A 149.76.14.12
    ...