## Comunicare client-server UNIX

Intr-o arhitectură client-server, clientul trimite request-uri (cere
resurse) către server, iar acesta din urmă trimite înapoi un raspuns (cu
resursa).

Pasi urmați pentru a schimba mesaje folosind UDP la nivelul Transport
folosind API-ul de sockets sunt următorii:

1.  *Deschide un socket* unix în scopul de a permite comunicarea între
    procese/statii diferite folosind descriptori de fisiere (file
    descriptors) cu apelul *socket()*.
2.  *Asociaza o adresa* pentru *socketul deschis* cu apelul *bind()*. În
    general, folosim *bind()* atunci când dorim să așteptam datagrame pe
    un anumit port. Bind este chemat pe server pentru a specifica la ce
    port să lege socket-ul.
3.  *Trimite/Recepționeaza date* cu apelul *recvfrom()/sendto()*.
4.  *Închide socket* prin *close()*.

```
   +--------+            +--------+
   | Server |            | Client |
   +--------+            +--------+                 
       |                      |
       |                      |          Descriere:                                 
    socket()               socket()       socket() - creează un endpoint de comunicare
       |                      |           bind() - atașează o adresa unui socket
     bind()                   |           sendto() - trimite o datagrama
       |                      |           receive() - primește o datagramă
    recv_from() <--------  sendto()       close() - eliberează file descriptorul
       |                      |
    send_to()   -------->  recvfrom()
       |                      |
    close()                close()
```

O implementare simplă de client care trimite o datagramă:

```c
#include <stdio.h>     
#include <stdlib.h>     
#include <unistd.h>     
#include <string.h>     
#include <sys/types.h>     
#include <sys/socket.h>     
#include <arpa/inet.h>     
#include <netinet/in.h>     
    
#define PORT     50000     
#define MAXLINE 1024 

int main() { 

	int sockfd; 
	char buffer[MAXLINE]; 
	char *hello = "Hello from client"; 
	struct sockaddr_in	 servaddr; 

	// Creating socket file descriptor 
	if ( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) { 
		perror("socket creation failed"); 
		exit(EXIT_FAILURE); 
	} 

	memset(&servaddr, 0, sizeof(servaddr)); 
	
	// Filling server information 
	servaddr.sin_family = AF_INET; 
	servaddr.sin_port = htons(PORT); 
	servaddr.sin_addr.s_addr = INADDR_ANY; 
	
	int n, len; 
	
	sendto(sockfd, (const char *)hello, strlen(hello), 
		0, (const struct sockaddr *) &servaddr, 
			sizeof(servaddr)); 
	printf("Hello message sent.\n"); 
		
	n = recvfrom(sockfd, (char *)buffer, MAXLINE, 
				MSG_WAITALL, (struct sockaddr *) &servaddr, 
				(socklen_t *)&len); 
	buffer[n] = '\0'; 
	printf("Server : %s\n", buffer); 

	close(sockfd); 
	return 0; 
} 
```

Implementarea server-ului care primește această datagramă este următoarea:

```c
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/uio.h>
#include <unistd.h>

#define PORT     50000 
#define MAXLINE 1024

int main() { 

	int sockfd; 
	char buffer[MAXLINE]; 
	char *hello = "Hello from server"; 
	struct sockaddr_in servaddr, cliaddr; 
	
	// Creating socket file descriptor 
	if ( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) { 
		perror("socket creation failed"); 
		exit(EXIT_FAILURE); 
	} 
	
	memset(&servaddr, 0, sizeof(servaddr)); 
	memset(&cliaddr, 0, sizeof(cliaddr)); 
	
	// Filling server information 
	servaddr.sin_family = AF_INET; // IPv4 
    servaddr.sin_addr.s_addr = INADDR_ANY; // INADDR_ANY = 0.0.0.0
	servaddr.sin_port = htons(PORT); 
	
	// Bind the socket with the server address 
	if ( bind(sockfd, (const struct sockaddr *)&servaddr, 
			sizeof(servaddr)) < 0 ) { 
		perror("bind failed"); 
		exit(EXIT_FAILURE); 
	} 
	
	int n; 
	// len is an unsinged int value/result, but it is recomanded
	// to use the socklen_t type
	socklen_t len = sizeof(cliaddr); 

	n = recvfrom(sockfd, (char *)buffer, MAXLINE, 
				MSG_WAITALL, (struct sockaddr *) &cliaddr, 
				&len); 
	buffer[n] = '\0';
	printf("Client : %s\n", buffer); 
	sendto(sockfd, (const char *)hello, strlen(hello), 
		0, (const struct sockaddr *) &cliaddr, 
			len); 
	printf("Hello message sent.\n"); 
	
	return 0; 
} 
```
