# Implementarea ferestrei glisante

Fereastra glisantă poate fi implementată sub forma unei liste. Lista va conține
toate segmentele. Vom folosi `min_seq` si `max_seq` pentru a determina pozitia
ferestrei.  Ne vom baza pe o implementare de listă simplu inlanțuită.

```c
/* List entry */
struct cel{
  /* Pointer catre segment */
  void* info;
  /* Dimensiunea segmentului header + date */
  int info_len;
  /* Numar de secventa segment */
  int seq;
  struct cel* next;
};

typedef struct cel list_entry;

/* Window as a list */
typedef struct {
  int size;
  int max_seq;
  list_entry* head;
}list;

/* Adauga in lista un segment. In info este stocat segmentul, len este dimensiunea
   segmentului si seq numarul de secventa */
void add_list_elem(list* window, void* segment, int len, int seq);
void display_list_seq(list* window);
list* create_list(int max_size);
```

Vizual, implementarea arata astfel:

![](images/window_representation.png)

> Slot-urile gri pot conține sau nu segmente.

Vom presupune ca am determinat `window_size = max_seq - min_seq` în funcție de BDP.
Atât la început, cât și la retransmisie, în implementarea de Go-Back-N vom
transmite toata fereastra.

```c
/* Functia trimite toate pachetele din fereastra */
int send_window(list *window) {
    list_entry* e = window->head;
    while(e != NULL && e->seq < window->max_seq)
        sendto(sockfd, e->info, e->info_len, &serveraddr, sizeof(serveraddr));
}
```

Ce facem atunci când primim un ACK cu numărul de secvență seq? Etichetăm
segmentul ca fiind primit prin faptul ca îl scoatem din listă.

```c
/* Scoate din lista segmentele care au fost primite */
int update_left_window(list *window, int seq) {
    list_entry* e = window->head;
    list_entry *prev = NULL;
    int count = 0;

    while(e != NULL && e->seq <= seq) {
        prev = e;
        e = e->next;
        free_entry(e);
        count++;
    }

    l->head = e;
    return count;
}
```

Acum ca fereastra s-a eliberat de count segmente. Noul `max_seq = max_seq + count`, așa
că trebuie să mutăm fereastra la dreapta și să trimitem urmatoarele segmente.

```c
int send_new_segments(list *window, int new_max_seq) {
    list_entry* e = window->head;
    /* Sarim peste segmentele deja trimise */
    while(e != NULL && e->seq <= window->max_seq)
        e = e->next;

    if (e != NULL) {
        /* Trimitem urmatoarele segmente pana la new_max_seq */
        while(e != NULL && e->seq <= new_max_seq) {
            e = e->next;
            sendto(sockfd, e->info, e->info_len, &serveraddr, sizeof(serveraddr));
        }
    }
    window->max_seq = new_max_seq;
}
```
